@@ -1,8 +1,8 @@
 import matplotlib as mpl
-mpl.use("agg")  # use most conservative backend
+mpl.use("agg")
 from .mesh import Mesh
 from .field import Field
-from .read import read_oommf_file
+from .read import read
 
 
 def test():
