@@ -1,4 +1,5 @@
 import random
+import itertools
 import numpy as np
 import matplotlib.pyplot as plt
 import joommfutil.typesystem as ts
@@ -64,7 +65,7 @@ class Mesh:
     >>> import discretisedfield as df
     >>> p1 = (-25, 3, 0)
     >>> p2 = (25, 6, 1)
-    >>> cell = (5, 3, 0.3)
+    >>> cell = (5, 3, 0.4)
     >>> mesh = df.Mesh(p1=p1, p2=p2, cell=cell, name=name)
     Traceback (most recent call last):
         ...
@@ -77,19 +78,12 @@ class Mesh:
         self.cell = tuple(cell)
         self.name = name
 
-        # Is the length of any mesh domain edge zero?
+        # Is the length of any mesh domain edges zero?
         for i, li in enumerate(self.l):
             if li == 0:
                 msg = "Mesh domain edge length is zero (l[{}]==0).".format(i)
                 raise ValueError(msg)
 
-        # Is the discretisation cell greater than the mesh domain?
-        for i, (li, celli) in enumerate(zip(self.l, self.cell)):
-            if celli > li:
-                msg = ("Discretisation cell is greater than the mesh domain: "
-                       "cell[{0}] > abs(p2[{0}]-p1[{0}]).").format(i)
-                raise ValueError(msg)
-
         # Is the mesh domain not an aggregate of discretisation cells?
         tol = 1e-12  # picometre tolerance
         for i, (li, celli) in enumerate(zip(self.l, self.cell)):
@@ -133,7 +127,7 @@ class Mesh:
         .. seealso:: :py:func:`~discretisedfield.Mesh.pmax`
 
         """
-        return tuple(min(coords) for coords in zip(self.p1, self.p2))
+        return tuple(map(min, zip(self.p1, self.p2)))
 
     @property
     def pmax(self):
@@ -170,7 +164,7 @@ class Mesh:
         .. seealso:: :py:func:`~discretisedfield.Mesh.pmin`
 
         """
-        return tuple(max(coords) for coords in zip(self.p1, self.p2))
+        return tuple(map(max, zip(self.p1, self.p2)))
 
     @property
     def l(self):
@@ -244,7 +238,7 @@ class Mesh:
     def centre(self):
         """Mesh domain centre point.
 
-        This point does not necessarily coincides with the
+        This point does not necessarily coincide with the
         discretisation cell centre. It is computed as the middle point
         between minimum and maximum coordinate :math:`p_{c}^{i} =
         p_\\text{min}^{i} + 0.5l^{i}`, where :math:`p_\\text{min}^{i}`
@@ -308,10 +302,8 @@ class Mesh:
         .. seealso:: :py:func:`~discretisedfield.Mesh.coordinates`
 
         """
-        for k in range(self.n[2]):
-            for j in range(self.n[1]):
-                for i in range(self.n[0]):
-                    yield (i, j, k)
+        for index in itertools.product(*map(range, reversed(self.n))):
+            yield tuple(reversed(index))
 
     @property
     def coordinates(self):
@@ -346,8 +338,8 @@ class Mesh:
         .. seealso:: :py:func:`~discretisedfield.Mesh.indices`
 
         """
-        for i in self.indices:
-            yield self.index2point(i)
+        for index in self.indices:
+            yield self.index2point(index)
 
     def __repr__(self):
         """Mesh representation.
@@ -379,12 +371,12 @@ class Mesh:
                 "name=\"{}\")").format(self.p1, self.p2, self.cell, self.name)
 
     def random_point(self):
-        """Generate the random point inside the mesh.
+        """Generate the random point belonging to the mesh.
 
         Returns
         -------
         tuple (3,)
-            Coordinates of a random point inside the mesh
+            Coordinates of a random point inside that belongs to the mesh
             :math:`(x_\\text{rand}, y_\\text{rand}, z_\\text{rand})`.
 
         Example
@@ -402,15 +394,15 @@ class Mesh:
         .. note::
 
            In the example, ellipsis is used instead of an exact tuple
-           because the result differs each time the random_point
-           command command is run.
+           because the result differs each time the ``random_point``
+           is called.
 
         """
         return tuple(pmini+random.random()*li
                      for pmini, li in zip(self.pmin, self.l))
 
     def index2point(self, index):
-        """Convert the cell index to its coordinate.
+        """Convert the cell index to its centre's coordinate.
 
         Parameters
         ----------
@@ -533,7 +525,7 @@ class Mesh:
                 raise ValueError(msg)
 
     def line(self, p1, p2, n=100):
-        """Line intersection generator.
+        """Line generator.
 
         Given two points :math:`p_{1}` and :math:`p_{2}`, :math:`n`
         position vectors are generated.
@@ -543,8 +535,8 @@ class Mesh:
            \\mathbf{r}_{i} = i\\frac{\\mathbf{p}_{2} -
            \\mathbf{p}_{1}}{n-1}
 
-        and this method yields :math:`|\\mathbf{r}_{i}|` and
-        :math:`\\mathbf{r}_{i}` in :math:`n` iterations.
+        and this method yields :math:`\\mathbf{r}_{i}` in :math:`n`
+        iterations.
 
         Parameters
         ----------
@@ -555,8 +547,8 @@ class Mesh:
 
         Yields
         ------
-            tuple of (float, tuple)
-                (:math:`|\\mathbf{r}_{i}|` and :math:`\\mathbf{r}_{i}`)
+            tuple
+                :math:`\\mathbf{r}_{i}`
 
         Raises
         ------
@@ -571,7 +563,7 @@ class Mesh:
         >>> cell = (1, 1, 1)
         >>> mesh = df.Mesh(p1=p1, p2=p2, cell=cell)
         >>> tuple(mesh.line(p1=(0, 0, 0), p2=(2, 0, 0), n=2))
-        ((0.0, (0.0, 0.0, 0.0)), (2.0, (2.0, 0.0, 0.0)))
+        ((0.0, 0.0, 0.0), (2.0, 0.0, 0.0))
 
         """
         self._isoutside(p1)
@@ -580,7 +572,33 @@ class Mesh:
         p1, p2 = np.array(p1), np.array(p2)
         dl = (p2-p1) / (n-1)
         for i in range(n):
-            yield np.linalg.norm(i*dl), tuple(p1+i*dl)
+            yield tuple(p1+i*dl)
+
+    def plane(self, *args, x=None, y=None, z=None, n=None):
+        info = dfu.plane_info(*args, x=x, y=y, z=z)
+
+        if info["point"] is None:
+            info["point"] = self.centre[info["slice"]]
+        else:
+            test_point = list(self.centre)
+            test_point[info["slice"]] = info["point"]
+            self._isoutside(test_point)
+
+        if n is None:
+            n = (self.n[info["haxis"]], self.n[info["vaxis"]])
+
+        dhaxis, dvaxis = self.l[info["haxis"]]/n[0], self.l[info["vaxis"]]/n[1]
+        haxis = np.linspace(self.pmin[info["haxis"]]+dhaxis/2,
+                            self.pmax[info["haxis"]]-dhaxis/2, n[0])
+        vaxis = np.linspace(self.pmin[info["vaxis"]]+dvaxis/2,
+                            self.pmax[info["vaxis"]]-dvaxis/2, n[1])
+
+        for x, y in itertools.product(haxis, vaxis):
+            point = 3*[None]
+            point[info["slice"]] = info["point"]
+            point[info["haxis"]] = x
+            point[info["vaxis"]] = y
+            yield tuple(point)
 
     def plot(self):
         """Plots the mesh domain and the discretisation cell.
@@ -599,8 +617,6 @@ class Mesh:
         dfu.plot_box(ax, self.pmin, cell_point, "r-", linewidth=1)
         ax.set(xlabel=r"$x$", ylabel=r"$y$", zlabel=r"$z$")
 
-        return fig
-
     def _ipython_display_(self):
         """Jupyter notebook mesh representation.
 
@@ -608,7 +624,7 @@ class Mesh:
         :py:func:`~discretisedfield.Mesh.plot`.
 
         """
-        return self.plot()  # pragma: no cover
+        self.plot()  # pragma: no cover
 
     @property
     def _script(self):
